﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using TFC.NET.Entities;
using TFC.NET.Business;
using TFC.NET.Master;
using System.IO;
using System.Reflection;

namespace TFC.NET.Control
{
    public partial class PublicationTools : System.Web.UI.UserControl
    {
        Decimal _userTo;

        /// <summary>
        /// Defines a publication into another profile
        /// </summary>
        public Decimal UserTo
        {

            get
            {
                // look for current page in ViewState
                object o = this.ViewState["_PubUser"];
                if (o == null)
                {

                    if (string.IsNullOrEmpty(Request["user"]) || !Decimal.TryParse(Request["user"], out _userTo))
                    {
                        throw new WrongDataException("No se pudo recuperar el usuario");
                    }
                    else
                    {
                        this.ViewState["_PubUser"] = _userTo;
                        return _userTo;
                    }
                }
                else
                {
                    return (Decimal)o;
                }
            }

            set
            {
                this.ViewState["_PubUser"] = value;
            }
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                Private privateFunctions = new Private();
                User user = new User();
                Session.Remove(user.SessionID + "_pubimg");
                try
                {

                    privateFunctions.isLoggedSession(ref user);
                    List<Rule> ruleList = new List<Rule>();

                    if (GlobalBS.GetRulesFromUser(user, ruleList))
                    {
                        ruleList.RemoveAll(delegate(Rule r) { return r.RuleTypeID == RuleType.Images; });
                        ruleList.Add(new Rule() { RuleName = "Público", RuleID = -1 });
                        drpRule.DataSource = ruleList;
                        drpRule.DataTextField = "RuleName";
                        drpRule.DataValueField = "RuleID";
                        drpRule.DataBind();
                    }
                }
                catch (WrongDataException ex)
                {
                    //Script register to show exception info
                    ScriptManager.RegisterStartupScript(this, this.GetType(), "showMsg", string.Format(@"jsAlert('{0}');", ex.Message), true);
                    responseTxt.Text = ex.Message;
                    return;
                }
                catch (Exception ex)
                {
                    //Script register to show exception info
                    ScriptManager.RegisterStartupScript(this, this.GetType(), "showMsg", @"jsError('Lo sentimos pero ha ocurrido un error inexperado');", true);

                    responseTxt.Text = "Lo sentimos pero ha ocurrido un error inexperado";

                    Utilities.LogException(Path.GetFileName(Request.Path),
                                MethodInfo.GetCurrentMethod().Name,
                                ex);
                    return;
                }
            }
        }

        /// <summary>
        /// Event handler for publication button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnPublication_Click(object sender, EventArgs e)
        {

            Private privateFunctions = new Private();
            User user = new User();
            try
            {

                privateFunctions.isLoggedSession(ref user);

                string msg = Utilities.StripHtml(txtpublication.Text, true);
                Picture pic = null;
                if (Session[user.SessionID + "_pubimg"] != null)
                {
                    pic = (Picture)Session[user.SessionID + "_pubimg"];

                    pic.PicDesc = txtimage.Text;
                }
                if (!string.IsNullOrEmpty(msg))
                {

                    Entities.Rule rule = new Rule()
                    {
                        RuleID = Decimal.Parse(drpRule.SelectedValue),
                        UserID = user.UserID
                    };

                    if (GlobalBS.InsertNewPublication(user, pic, rule, -1, UserTo, msg))
                    {
                        txtimage.Text = "";
                        txtpublication.Text = "";
                        AsyncFileUpload.ClearAllFilesFromPersistedStore();
                        //ScriptManager.RegisterStartupScript(this, this.GetType(), "hideFancybox", @"setTimeout(function(){$.fancybox.close();document.location.refresh();},5000)", true);
                        Session.Remove(user.SessionID + "_pubimg");

                        if (user.UserID != UserTo)
                        {
                            Context.Response.Redirect(Request.Url.ToString());
                        }
                        else
                        {
                            Context.Response.Redirect("Home.aspx");
                        }
                    }
                }
                else
                {
                    responseTxt.Text = "Por favor, introduzca un texto para su publicación";
                }
            }
            catch (WrongDataException ex)
            {
                //Script register to show exception info
                ScriptManager.RegisterStartupScript(this, this.GetType(), "showMsg", string.Format(@"jsAlert('{0}');", ex.Message), true);
                responseTxt.Text = ex.Message;
                return;
            }
            catch (Exception ex)
            {
                //Script register to show exception info
                ScriptManager.RegisterStartupScript(this, this.GetType(), "showMsg", @"jsError('Lo sentimos pero ha ocurrido un error inexperado');", true);

                responseTxt.Text = "Lo sentimos pero ha ocurrido un error inexperado";
                
                Utilities.LogException(Path.GetFileName(Request.Path),
                            MethodInfo.GetCurrentMethod().Name,
                            ex);
                return;
            }
        }

        /// <summary>
        /// Event handler for file upload completed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void AsyncFileUpload_UploadedComplete(Object sender, AjaxControlToolkit.AsyncFileUploadEventArgs e)
        {

            if (AsyncFileUpload.HasFile)
            {
                try
                {

                    Private privateFunctions = new Private();
                    User user = new User();
           
                    privateFunctions.isLoggedSession(ref user);

                    Picture pic = null;
                    HttpPostedFile file = AsyncFileUpload.PostedFile;
                    string imgmsg = "";
                    pic = new Picture() { AlbumID = 0, PicDate = DateTime.Now, PicDesc = txtimage.Text };
                    if (!Utilities.UploadImageFromUser(file, ref user, ref pic, ref imgmsg))
                    {
                        responseTxt.Text = imgmsg;
                        return;
                    }

                    Session.Add(user.SessionID + "_pubimg", pic);

                    imgUploaded.ImageUrl = string.Format("~/AjaxHandler.ashx?fn=ResizeRatio&size=pub&src=\\userData\\{0}", pic.PicPath);
                    imgUploaded.Visible = true;


                }
                catch (Exception ex)
                {
                    Utilities.LogException(Path.GetFileName(Request.Path),
                        MethodInfo.GetCurrentMethod().Name,
                        ex);
                    responseTxt.Text = "Ha ocurrido un error inexperado al intentar subir su imagen. Por favor, inténtelo más tarde.";
                }
            }

            responseTxt.Text = "Imagen subida correctamente";
        }

        /// <summary>
        /// Event handler for file upload error
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void AsyncFileUpload_UploadedFileError(Object sender, AjaxControlToolkit.AsyncFileUploadEventArgs e) 
        {

            responseTxt.Text = "Lo sentimos pero ocurrió un error al procesar su imagen"; 
            Utilities.LogException("AjaxLoadingImage",MethodInfo.GetCurrentMethod().ToString(), new WrongDataException(e.StatusMessage));
            

        }

    }
}