﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Text;
using TFC.NET.Business;
using TFC.NET.Entities;
using System.Reflection;
using System.IO;

namespace TFC.NET
{
    public partial class Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            ///TODO: 
            /// Se podría guardar en cookie el id de sesión obtenido y después
            /// aquí se podría hacer una primera comprobación de la anterior sesión
            /// que en caso de existir ya la cogeríamos directa para entrar
        }

        /// <summary>
        /// EventHandler of login button click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnLogin_Click(object sender, EventArgs e)
        {
            StringBuilder msg = new StringBuilder("");
            if (validateForm(ref msg))
            {
                //Creates user object
                Entities.User user = new Entities.User()
                {
                    UserLogin = userinput.Text,
                    UserPass = Utilities.EncodeMD5(passinput.Text),
                    StatuID = Status.Pending
                };

                try
                {
                    // Try to login by using this data
                    if (DefaultBS.ProcessLoginForm(user))
                    {
                        // Login active and sets into sessions keeper
                        Global.sessionsTable.Insert(user.SessionID, 
                                                user,
                                                null,
                                                System.Web.Caching.Cache.NoAbsoluteExpiration, 
                                                TimeSpan.FromMinutes(20));
                        //Saves on cookies the session
                        Context.Response.Cookies.Add(new HttpCookie("socialMe") { 
                            Expires = DateTime.Now.AddDays(1),
                            HttpOnly = true,
                            Value = user.SessionID,
                            Name = "session"                        
                        });

                        // All goes fine and then can enter to application
                        Response.Redirect("Home.aspx");
                    }
                    else
                    {

                        lblMsg.Text = "Ha ocurrido un error al procesar su petición de login. Por favor pruebe más tarde";
                        Utilities.LogException(Path.GetFileName(Request.Path),
                            MethodInfo.GetCurrentMethod().Name,
                            new Exception(string.Concat(msg.ToString(), "No ha sido posible completar el login")));
                    }
                }
                catch (WrongDataException ex)
                {
                    lblMsg.Text = ex.Message;
                }

            }
            else
            {
                lblMsg.Text = msg.ToString();
            }

            //Script register to restore button functionality and show any issue or message
            ScriptManager.RegisterStartupScript(this, this.GetType(), "showMsg", @"$('.loginform .msg').show();
                                                                                       $('.preventDouble').enableSubmit();
                                                                                        setTimeout(function(){$('.loginform .msg').hide();},5000);", true);
        }

        /// <summary>
        /// Method to validate login form
        /// </summary>
        /// <param name="msg"></param>
        /// <returns></returns>
        private bool validateForm(ref StringBuilder msg)
        {
            if (userinput.Text == string.Empty ||
               userinput.Text == "usuario")
            {
                msg.Append("<p>Por favor, rellene su nombre</p>");
            }

            if (passinput.Text == string.Empty ||
                passinput.Text == "contraseña")
            {
                msg.Append("<p>Por favor, introduce una contraseña correcta</p>");
            }

            if (msg.Length > 0)
                return false;

            msg.AppendLine("Petición de Login:");
            msg.AppendLine(string.Format("User:{0}", userinput.Text));
            
            return true;
        }
    }
}