﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using TFC.NET.Master;
using TFC.NET.Entities;
using System.IO;
using System.Reflection;
using TFC.NET.Business;
using System.Text;

namespace TFC.NET
{
    public partial class UploadImage : System.Web.UI.Page
    {
        private string _profileImageUrl;
        private string _userSessionID;

        public string UserSessionID
        {
            get { return _userSessionID; }
            set { _userSessionID = value; }
        }

        public string ProfileImageUrl
        {
            get { return _profileImageUrl; }
            set { _profileImageUrl = value; }
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            Private privateFunctions = new Private();
            User user = new User();
            List<Rule> rules = new List<Rule>();
            try
            {

                if (privateFunctions.isLoggedSession(ref user))
                {
                    UserSessionID = user.SessionID;
                    if (!Page.IsPostBack)
                    {

                        Session.Remove(user.SessionID + "_newimage");

                        try
                        {
                            LoadAlbums(ref user);
                            LoadRules(ref user);

                        }
                        catch (WrongDataException ex)
                        {
                            //Script register to show exception info
                            ScriptManager.RegisterStartupScript(this, this.GetType(), "showMsg", string.Format(@"jsError('{0}');", ex.Message), true);
                            return;
                        }
                        catch (Exception ex)
                        {
                            //Script register to show exception info
                            ScriptManager.RegisterStartupScript(this, this.GetType(), "showMsg", @"jsError('Lo sentimos pero ha ocurrido un error inexperado');", true);
                            Utilities.LogException(Path.GetFileName(Request.Path),
                                        MethodInfo.GetCurrentMethod().Name,
                                        ex);
                            return;
                        }
                    }
                    else
                    {
                        string eventArgs = Request["__EVENTARGUMENT"];
                        if (eventArgs.StartsWith("LoadAlbum"))
                        {
                            Decimal albumid;
                            if (Decimal.TryParse(eventArgs.Split(';')[1], out albumid))
                            {
                                Album album = new Album()
                                {
                                    AlbumID = albumid
                                };

                                List<Picture> pictureList = new List<Picture>();

                                if (GlobalBS.GetPicturesFromAlbum(ref album, ref pictureList))
                                {
                                    rptPictures.DataSource = pictureList;
                                    rptPictures.DataBind();
                                }

                            }
                            else
                            {
                                rptPictures.DataSource = null;
                                rptPictures.DataBind();

                            }
                        }
                        else if (eventArgs.StartsWith("RefreshAlbums"))
                        {
                            LoadAlbums(ref user);
                            rptPictures.DataSource = null;
                            rptPictures.DataBind();
                        }
                        ScriptManager.RegisterStartupScript(this, this.GetType(), "showMsg", @"DOMLoadObjects();$('#pictures').scrollable();", true);
                    }
                }
            }
            catch (WrongDataException ex)
            {
                //Script register to show exception info
                ScriptManager.RegisterStartupScript(this, this.GetType(), "showMsg", string.Format(@"jsAlert('{0}');", ex.Message), true);
                return;
            }
            catch (Exception ex)
            {
                //Script register to show exception info
                ScriptManager.RegisterStartupScript(this, this.GetType(), "showMsg", @"jsError('Lo sentimos pero ha ocurrido un error inexperado');", true);

                Utilities.LogException(Path.GetFileName(Request.Path),
                            MethodInfo.GetCurrentMethod().Name,
                            ex);
                return;
            }
        }

        /// <summary>
        /// Load albums into containers
        /// </summary>
        /// <param name="user"></param>
        private void LoadAlbums(ref User user)
        {
            List<Album> albumList = new List<Album>();
            if (GlobalBS.GetAlbums(user, user, ref albumList))
            {
                rptAlbum.DataSource = albumList;
                rptAlbum.DataBind();

            }
            albumList.Add(new Album() { AlbumID = -1, AlbumName = "Nuevo álbum" });
            albumList.RemoveAll(delegate(Album a) { return a.IsProfile || a.IsPublications; });
            drpAlbum.DataSource = albumList;
            drpAlbum.DataTextField = "AlbumName";
            drpAlbum.DataValueField = "AlbumID";
            drpAlbum.DataBind();

            
        }

        private void LoadRules(ref User user)
        {
            List<Rule> ruleList = new List<Rule>();

            if (GlobalBS.GetRulesFromUser(user, ruleList))
            {
                ruleList.RemoveAll(delegate(Rule r) { return r.RuleTypeID == RuleType.Publications; });
                ruleList.Add(new Rule() { RuleName = "Público", RuleID = -1 });
                drpRule.DataSource = ruleList;
                drpRule.DataTextField = "RuleName";
                drpRule.DataValueField = "RuleID";
                drpRule.DataBind();
            }
        }



        /// <summary>
        /// Event handler for file upload completed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void AsyncFileUpload_UploadedComplete(Object sender, AjaxControlToolkit.AsyncFileUploadEventArgs e)
        {

            if (AsyncFileUpload.HasFile)
            {
                try
                {

                    Private privateFunctions = new Private();
                    User user = new User();

                    privateFunctions.isLoggedSession(ref user);

                    Picture pic = null;
                    HttpPostedFile file = AsyncFileUpload.PostedFile;
                    string imgmsg = "";
                    pic = new Picture() { AlbumID = 0, PicDate = DateTime.Now };
                    if (!Utilities.UploadImageFromUser(file, ref user, ref pic, ref imgmsg))
                    {
                        txtprofileimg.Text = imgmsg;
                        return;
                    }

                    Session[user.SessionID + "_newimage"] = pic;

                }
                catch (Exception ex)
                {
                    Utilities.LogException(Path.GetFileName(Request.Path),
                        MethodInfo.GetCurrentMethod().Name,
                        ex);
                    txtprofileimg.Text = "Ha ocurrido un error inexperado al intentar subir su imagen. Por favor, inténtelo más tarde.";
                }
            }

            txtprofileimg.Text = "Imagen subida correctamente";
        }

        /// <summary>
        /// Event handler for file upload error
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void AsyncFileUpload_UploadedFileError(Object sender, AjaxControlToolkit.AsyncFileUploadEventArgs e)
        {

            txtprofileimg.Text = "Lo sentimos pero ocurrió un error al procesar su imagen";
            Utilities.LogException("AjaxLoadingImage", MethodInfo.GetCurrentMethod().ToString(), new WrongDataException(e.StatusMessage));


        }

        /// <summary>
        /// Event handler for save image click button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnSaveImage_Click(object sender, EventArgs e)
        {
            try
            {
                StringBuilder msg = new StringBuilder();
                Private privateFunctions = new Private();
                User user = new User();

                privateFunctions.isLoggedSession(ref user);

                if (_validateForm(user, ref msg))
                {
                    Decimal albumid;
                    Album album = new Album() { AlbumName = txtAlbumName.Text, AlbumDesc = txtAlbumDesc.Text, UserID = user.UserID, RuleID = Decimal.Parse(drpRule.SelectedValue) };
                    if (drpAlbum.SelectedValue == "-1")
                    {
                        if (!GlobalBS.InsertNewAlbum(ref album))
                        {
                            throw new WrongDataException("Se produjo un error al intentar crear el álbum");
                        }

                        albumid = album.AlbumID;

                    }
                    else
                    {
                        albumid = Decimal.Parse(drpAlbum.SelectedValue);
                    }

                    Picture pic = null;
                    if (Session[user.SessionID + "_newimage"] != null)
                    {
                        pic = (Picture)Session[user.SessionID + "_newimage"];
                        pic.AlbumID = albumid;
                        pic.PicName = txtname.Text;
                        pic.PicDesc = txtdesc.Text;
                    }

                    if (pic != null)
                    {

                        if (GlobalBS.InsertNewImage(user, pic))
                        {
                            Session.Remove(user.SessionID + "_newimage");
                            LoadAlbums(ref user);
                            rptPictures.DataSource = null;
                            rptPictures.DataBind();
                            txtdesc.Text = txtname.Text = txtAlbumDesc.Text = txtAlbumName.Text = string.Empty;
                        }
                        else
                        {
                            throw new WrongDataException("No se pudo almacenar la imagen en el sistema");
                        }
                    }
                }
            }
            catch (WrongDataException ex)
            {
                //Script register to show exception info
                ScriptManager.RegisterStartupScript(this, this.GetType(), "showMsg",  string.Format(@"jsAlert('{0}');$('.preventDouble').enableSubmit();", ex.Message), true);
                return;
            }
            catch (Exception ex)
            {
                //Script register to show exception info
                ScriptManager.RegisterStartupScript(this, this.GetType(), "showMsg", @"jsError('Lo sentimos pero ha ocurrido un error inexperado');", true);

                Utilities.LogException(Path.GetFileName(Request.Path),
                            MethodInfo.GetCurrentMethod().Name,
                            ex);
                return;
            }
        }

        private bool _validateForm(User u, ref StringBuilder msg)
        {
            if (string.IsNullOrEmpty(txtdesc.Text) && txtdesc.Text.Length > 255)
            {
                msg.Append("La descripción introducida contiene más carácteres del máximo (255)");
            }

            DateTime picdate;
            if (string.IsNullOrEmpty(picdateinput.Text) && !DateTime.TryParse(picdateinput.Text, out picdate) && picdate > DateTime.Now)
            {
                msg.Append("La fecha introducida no es válida");
            }

            if (Session[u.SessionID + "_newimage"] == null)
            {
                msg.Append("Lo sentimos, pero parece que no adjuntó ninguna imagen");
            }

            if (msg.Length > 0)
                return false;

            msg.AppendLine("Nueva imagen:");
            msg.AppendLine(string.Format("Descripción:{0}", txtdesc.Text));
            msg.AppendLine(string.Format("Usuario:{0}", u.UserID));
            return true;
        }

    }
}